function add_itemdrop_counter(name, f)
	add_printer("item drop: " .. name, function ()
		local c = count(item_name)
		local msg = f(c)
		if msg then
			text = text .. "<center>" .. msg .. "</center>"
		end
	end)
end

do
	local localtable = table
	local localsession = session
	local localmake_href = make_href

	local zone_checks_ascension = {}
	local zone_check_added_ascension = false
	function add_ascension_zone_check(zoneid, check)
		if not zone_check_added_ascension then
			add_interceptor("/adventure.php", function()
				if get_character_state("enable adventure warnings") ~= "yes" then return end
				if ascensionstatus() == "Aftercore" then return end
				local zid = tonumber(params.snarfblat)
				for xidx, xf in pairs(zone_checks_ascension[zid] or {}) do
					-- NB: Zone warnings are identified by zone and order added. This is a bit tricky, it changes if new ones are added.
					local x = xf()
					if x then
						local a, b = intercept_warning(x, "zonecheck-ascension-" .. zid .. "-" .. xidx)
						if a then return a, b end
					end
				end
			end)
			zone_check_added_ascension = true
		end
		if not zone_checks_ascension[zoneid] then zone_checks_ascension[zoneid] = {} end
		localtable.insert(zone_checks_ascension[zoneid], check)
	end

	local zone_checks_aftercore = {}
	local zone_check_added_aftercore = false
	function add_aftercore_zone_check(zoneid, check)
		if not zone_check_added_aftercore then
			add_interceptor("/adventure.php", function()
				if get_character_state("enable adventure warnings") ~= "yes" then return end
				if ascensionstatus() ~= "Aftercore" then return end
				local zid = tonumber(params.snarfblat)
				for xidx, xf in pairs(zone_checks_aftercore[zid] or {}) do
					-- NB: Zone warnings are identified by zone and order added. This is a bit tricky, it changes if new ones are added.
					local x = xf()
					if x then
						local a, b = intercept_warning(x, "zonecheck-aftercore-" .. zid .. "-" .. xidx)
						if a then return a, b end
					end
				end
			end)
			zone_check_added_aftercore = true
		end
		if not zone_checks_aftercore[zoneid] then zone_checks_aftercore[zoneid] = {} end
		localtable.insert(zone_checks_aftercore[zoneid], check)
	end

	local zone_checks_always = {}
	local zone_check_added_always = false
	function add_always_zone_check(zoneid, check)
		if not zone_check_added_always then
			add_interceptor("/adventure.php", function()
				if get_character_state("enable adventure warnings") ~= "yes" then return end
				local zid = tonumber(params.snarfblat)
				for xidx, xf in pairs(zone_checks_always[zid] or {}) do
					-- NB: Zone warnings are identified by zone and order added. This is a bit tricky, it changes if new ones are added.
					local x = xf()
					if x then
						local a, b = intercept_warning(x, "zonecheck-always-" .. zid .. "-" .. xidx)
						if a then return a, b end
					end
				end
			end)
			zone_check_added_always = true
		end
		if not zone_checks_always[zoneid] then zone_checks_always[zoneid] = {} end
		localtable.insert(zone_checks_always[zoneid], check)
	end

	local function check_adventure_warning(f)
		if character["enable adventure warnings"] ~= "yes" then return end
		local rets = { f() }
		if rets[1] then
			return intercept_warning(unpack(rets))
		end
	end

	function add_ascension_warning(filename, f)
		add_interceptor(filename, function()
			if ascensionstatus() == "Aftercore" then return end
			return check_adventure_warning(f)
		end)
	end

	function add_aftercore_warning(filename, f)
		add_interceptor(filename, function()
			if ascensionstatus() ~= "Aftercore" then return end
			return check_adventure_warning(f)
		end)
	end

	function add_always_warning(filename, f)
		add_interceptor(filename, function()
			return check_adventure_warning(f)
		end)
	end

	function add_ascension_adventure_warning(f)
		add_interceptor("/adventure.php", function()
			if ascensionstatus() == "Aftercore" then return end
			return check_adventure_warning(f)
		end)
		add_interceptor("/hiddencity.php", function()
			if ascensionstatus() == "Aftercore" then return end
			if params.which == nil then return end
			return check_adventure_warning(f)
		end)
	end

	function add_aftercore_adventure_warning(f)
		add_interceptor("/adventure.php", function()
			if ascensionstatus() ~= "Aftercore" then return end
			return check_adventure_warning(f)
		end)
		add_interceptor("/hiddencity.php", function()
			if ascensionstatus() ~= "Aftercore" then return end
			if params.which == nil then return end
			return check_adventure_warning(f)
		end)
	end

	function add_always_adventure_warning(f)
		add_interceptor("/adventure.php", function()
			return check_adventure_warning(f)
		end)
		add_interceptor("/hiddencity.php", function()
			if params.which == nil then return end
			return check_adventure_warning(f)
		end)
	end

	local added_automation_handler = false
	local automation_scripts = {}

	function add_automation_script(x, f)
		if not added_automation_handler then
			add_interceptor("/kolproxy-automation-script", function()
				local scriptname = params["automation-script"]
				if not scriptname or params.pwd ~= localsession.pwd then
					error "Error: Invalid settings when running automation script"
				end
				if automation_scripts[scriptname] then
					return automation_scripts[scriptname]()
				end
				error "Error: Automation script not found"
			end)
			added_automation_handler = true
		end
		automation_scripts[x] = f
		return function(tbl)
			tbl["automation-script"] = x
			return localmake_href("/kolproxy-automation-script", tbl)
		end
	end
end


function set_ascension_turn_counter(name, length)
	local tbl = ascension["turn counters"] or {}
	table.insert(tbl, { name = name, turn = turnsthisrun(), length = length })
	ascension["turn counters"] = tbl
end
